function Rate = LOSO(fea, label, subject, d, r, g)
% This function calculates the recognition rate using Leave-One-Subject-Out cross validation.
% 
% input:
% --fea:       size: m x c. m - samples, c - feature.
% --label:     size: m x 1. 
% --subject:   size: m x 1, indicating the subject id of each sample for loso.
% libsvm kernel_type: polynomial:  (gamma*u'*v + coef0)^degree
% --d:         parameter degree.
% --r:         parameter coef0.
% --g:         parameter gamma.
%
% output:
% --Rate:      Recognition rate of LOSO cross valiadation.

    % get feature samples
    nSamples = size(fea,1);

    % get subject class 
    subject_set = unique(subject);
    subject_number = size(subject_set,1);
    rates = zeros(subject_number,1);

    % LOSO loop
    for s = 1 : subject_number
        ind_Test = find(subject==subject_set(s,1));
        ind = 1:nSamples;
        ind_Train = setdiff(ind,ind_Test);
        fea_Train = fea(ind_Train, :); 
        gnd_Train = label(ind_Train);
        fea_Test = fea(ind_Test, :);
        gnd_Test = label(ind_Test);

        % svm training/prediction using a polynomial kernel
        model = svmtrain(gnd_Train, fea_Train, ['-c 1 -t 1 -d ' num2str(d) ' -r ' num2str(r) ' -g ' num2str(g) ' -e 0.000001']);
        [predict_label, accuracy, dec_values] = svmpredict(gnd_Test,fea_Test,model);

        % record the recognized number of each subject
        rates(s) = accuracy(1) / 100 * size(predict_label, 1);
    end
    
    % get total recognition rate
    Rate = sum(rates) / nSamples * 100;
end