function W = buildGraph(data, metric, k)
% This function calculates the graph weight matrix of input data.
% 
% input:
% --data:      size: m x n x c. m - samples, n - normalized frames, c - feature.
% --metric:    type of  graph construction:
%              'Euclidean' for GraphSC-MDMO
%              'L1Graph' for Sparse MDMO
%              default: 'L1Graph'
% --k:         number of nearest neighbours in graph. default: 5
% 
% output:
% --W:         graph weight matrix. size: (m x n) x (m x n).

    tic;

    if (~exist('metric','var'))
       metric = 'L1Graph';
    end

    if (~exist('k','var'))
       k = 5;
    end
    
    %data
    [m,n,c] = size(data);
    
    disp(['Build graph start!']);

    % construct the graph using L1-norm graph
    if strcmp(metric, 'L1Graph') == 1
        
        disp(['Construct L1-normed graph...']);
        
        % parameter for l1 solver
        mode = 2;
        lambda = 0.008;

        W = zeros(m*n,m*n);
        for i = 1:m
            if mod(i,10) == 0
                disp(['Processing video :' num2str(i) '/' num2str(m)]);
            end

            % choose the ith video feature from data
            tmpfea = data(i,:,:);
            fea = reshape(tmpfea,n,c);

            % l1 optimization
            [vW,~] = L1Graph(fea',mode,lambda);

            % choose the max k similar points
            [~,index] = sort(vW,2,'descend');
            idx = index(:,1:k);
            nvW = zeros(n,n);
            for p = 1:n
                for q = 1:k
                    nvW(p,idx(p,q)) = 1;
                end
            end

            % put the sub-graph into W
            W((i-1)*n+1:i*n,(i-1)*n+1:i*n) = nvW;
        end

        % make W sparse and symmetry
        W = sparse(max(W,W'));
    else
        % construct graph using Euclidean metric in whole data space
        disp(['Construct Euclidean-based graph...']);
        
        % reshape data
        fea = zeros(m*n,c);
        for i = 1:m
            for j = 1:n
                fea((i-1)*n+j,1:c) = data(i,j,1:c);
            end
        end
            
        options = [];
        options.NeighborMode = 'KNN';
        options.k = k;
        options.WeightMode = 'Binary';
        W = constructW(fea,options);
    end 
    
    disp(['Build graph end!']);
    toc;
end

